@@ -17,27 +17,6 @@
 #include <zim/archive.h>
 #include <zim/item.h>
 
-// Specialization of std::hash needed for our unordered_map. Can be removed in c++14
-namespace std {
-  template <> struct hash<LogTag> {
-    size_t operator() (const LogTag &t) const { return size_t(t); }
-  };
-}
-
-// Specialization of std::hash needed for our unordered_map. Can be removed in c++14
-namespace std {
-  template <> struct hash<TestType> {
-    size_t operator() (const TestType &t) const { return size_t(t); }
-  };
-}
-
-// Specialization of std::hash needed for our unordered_map. Can be removed in c++14
-namespace std {
-  template <> struct hash<MsgId> {
-    size_t operator() (const MsgId &msgid) const { return size_t(msgid); }
-  };
-}
-
 namespace
 {
 
@@ -56,6 +35,7 @@ std::unordered_map<TestType, std::pair<LogTag, std::string>> errormapping = {
     { TestType::REDUNDANT,     {LogTag::WARNING, "Redundant data found"}},
     { TestType::URL_INTERNAL,  {LogTag::ERROR, "Invalid internal links found"}},
     { TestType::URL_EXTERNAL,  {LogTag::ERROR, "Invalid external links found"}},
+    { TestType::URL_EMPTY,     {LogTag::WARNING, "Empty links found"}},
     { TestType::REDIRECT,      {LogTag::ERROR, "Redirect loop(s) exist"}},
 };
 
@@ -70,9 +50,9 @@ std::unordered_map<MsgId, MsgInfo> msgTable = {
   { MsgId::MAIN_PAGE,        { TestType::MAIN_PAGE, "Main Page Index stored in Archive Header: {{&main_page_index}}" } },
   { MsgId::EMPTY_ENTRY,      { TestType::EMPTY, "Entry {{&path}} is empty" } },
   { MsgId::OUTOFBOUNDS_LINK, { TestType::URL_INTERNAL, "{{&link}} is out of bounds. Article: {{&path}}" } },
-  { MsgId::EMPTY_LINKS,      { TestType::URL_INTERNAL, "Found {{&count}} empty links in article: {{&path}}" } },
   { MsgId::DANGLING_LINKS,   { TestType::URL_INTERNAL, "The following links:\n{{#links}}- {{&value}}\n{{/links}}({{&normalized_link}}) were not found in article {{&path}}" } },
   { MsgId::EXTERNAL_LINK,    { TestType::URL_EXTERNAL, "{{&link}} is an external dependence in article {{&path}}" } },
+  { MsgId::EMPTY_LINKS,      { TestType::URL_EMPTY, "Found {{&count}} empty links in article: {{&path}}" } },
   { MsgId::REDUNDANT_ITEMS,  { TestType::REDUNDANT, "{{&path1}} and {{&path2}}" } },
   { MsgId::METADATA,         { TestType::METADATA, "{{&error}}" } },
   { MsgId::REDIRECT_LOOP,    { TestType::REDIRECT, "Redirect loop exists from entry {{&entry_path}}\n"  } },
@@ -100,6 +80,7 @@ const char* toStr(TestType tt) {
     case TestType::REDUNDANT:    return "redundant";
     case TestType::URL_INTERNAL: return "url_internal";
     case TestType::URL_EXTERNAL: return "url_external";
+    case TestType::URL_EMPTY:    return "url_empty";
     case TestType::REDIRECT:     return "redirect";
     default:  throw std::logic_error("Invalid TestType");
   };
@@ -111,6 +92,11 @@ SortedMsgParams sortedMsgParams(const MsgParams& msgParams)
   return SortedMsgParams(msgParams.begin(), msgParams.end());
 }
 
+bool areAliases(const zim::Item& i1, const zim::Item& i2)
+{
+    return i1.getClusterIndex() == i2.getClusterIndex() && i1.getBlobIndex() == i2.getBlobIndex();
+}
+
 } // unnamed namespace
 
 namespace JSON
@@ -254,7 +240,7 @@ void test_checksum(zim::Archive& archive, ErrorLogger& reporter) {
     }
 }
 
-void test_integrity(const std::string& filename, ErrorLogger& reporter) {
+bool test_integrity(const std::string& filename, ErrorLogger& reporter) {
     reporter.infoMsg("[INFO] Verifying ZIM-archive structure integrity...");
     zim::IntegrityCheckList checks;
     checks.set(); // enable all checks (including checksum)
@@ -263,6 +249,7 @@ void test_integrity(const std::string& filename, ErrorLogger& reporter) {
     if (!result) {
         reporter.infoMsg("  [ERROR] ZIM file's low level structure is invalid");
     }
+    return result;
 }
 
 
@@ -485,16 +472,23 @@ void ArticleChecker::detect_redundant_articles()
         progress.report();
         auto l = it.second;
         while ( !l.empty() ) {
-            const auto e1 = archive.getEntryByPath(l.front());
+            // The way we have constructed `l`, e1 MUST BE an item
+            const auto e1 = archive.getEntryByPath(l.front()).getItem();
             l.pop_front();
             if ( !l.empty() ) {
-                // The way we have constructed `l`, e1 MUST BE an item
-                const std::string s1 = e1.getItem().getData();
+                std::unique_ptr<std::string> s1;
                 decltype(l) articlesDifferentFromE1;
                 for(auto other : l) {
-                    auto e2 = archive.getEntryByPath(other);
-                    std::string s2 = e2.getItem().getData();
-                    if (s1 != s2 ) {
+                    // The way we have constructed `l`, e2 MUST BE an item
+                    const auto e2 = archive.getEntryByPath(other).getItem();
+                    if (areAliases(e1, e2)) {
+                        continue;
+                    }
+                    if (!s1) {
+                        s1 = std::make_unique<std::string>(e1.getData());
+                    }
+                    std::string s2 = e2.getData();
+                    if (*s1 != s2 ) {
                         articlesDifferentFromE1.push_back(other);
                         continue;
                     }
