@@ -31,7 +31,6 @@
 #include <sys/stat.h>
 #include <iomanip>
 #include <vector>
-#include <codecvt>
 #include <unordered_map>
 
 #include "version.h"
@@ -49,7 +48,22 @@
 
 #define ERRORSDIR "_exceptions/"
 
-std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
+
+#ifdef _WIN32
+std::wstring utf8ToUtf16(const std::string& string) {
+  auto size = MultiByteToWideChar(CP_UTF8, 0,
+                string.c_str(), -1, nullptr, 0);
+  auto wdata = std::wstring(size, 0);
+  auto ret = MultiByteToWideChar(CP_UTF8, 0,
+                string.c_str(), -1, wdata.data(), size);
+  if (0 == ret) {
+    std::ostringstream oss;
+    oss << "Cannot convert string to wchar : " << GetLastError();
+    throw std::runtime_error(oss.str());
+  }
+  return wdata;
+}
+#endif
 
 inline static void createdir(const std::string &path, const std::string &base)
 {
@@ -59,7 +73,7 @@ inline static void createdir(const std::string &path, const std::string &base)
         if (position != std::string::npos) {
             std::string fulldir = base + SEPARATOR + path.substr(0, position);
             #if defined(_WIN32)
-            std::wstring wfulldir = converter.from_bytes(fulldir);
+            std::wstring wfulldir = utf8ToUtf16(fulldir);
             CreateDirectoryW(wfulldir.c_str(), NULL);
             #else
               ::mkdir(fulldir.c_str(), 0777);
@@ -89,6 +103,7 @@ class ZimDumper
     int listEntriesByNamespace(const std::string ns, bool details);
 
     zim::Entry getEntryByPath(const std::string &path);
+    zim::Entry getEntryByNsAndPath(char ns, const std::string &path);
     zim::Entry getEntry(zim::size_type idx);
 
     void dumpFiles(const std::string& directory, bool symlinkdump, std::function<bool (const char c)> nsfilter);
@@ -102,6 +117,11 @@ zim::Entry ZimDumper::getEntryByPath(const std::string& path)
     return m_archive.getEntryByPath(path);
 }
 
+zim::Entry ZimDumper::getEntryByNsAndPath(char ns, const std::string &path)
+{
+    return m_archive.getEntryByPathWithNamespace(ns, path);
+}
+
 zim::Entry ZimDumper::getEntry(zim::size_type idx)
 {
     return m_archive.getEntryByPath(idx);
@@ -209,7 +229,7 @@ int ZimDumper::listEntriesByNamespace(const std::string ns, bool details)
     return ret;
 }
 
-void write_to_error_directory(const std::string& base, const std::string relpath, const char *content, ssize_t size)
+void write_to_error_directory(const std::string& base, const std::string relpath, const char *content, size_t size)
 {
     createdir(ERRORSDIR, base);
     std::string url = relpath;
@@ -220,14 +240,14 @@ void write_to_error_directory(const std::string& base, const std::string relpath
 
 #ifdef _WIN32
     auto fullpath = std::string(base + ERRORSDIR + url);
-    std::wstring wpath = converter.from_bytes(fullpath);
+    std::wstring wpath = utf8ToUtf16(fullpath);
     auto fd = _wopen(wpath.c_str(), _O_WRONLY | _O_CREAT | _O_TRUNC, S_IWRITE);
 
     if (fd == -1) {
         std::cerr << "Error opening file " + fullpath + " cause: " + ::strerror(errno) << std::endl;
         return ;
     }
-    if (write(fd, content, size) != size) {
+    if ((size_t) write(fd, content, size) != size) {
       close(fd);
       std::cerr << "Failed writing: " << fullpath << " - " << ::strerror(errno) << std::endl;
     }
@@ -246,10 +266,10 @@ void write_to_error_directory(const std::string& base, const std::string relpath
 #endif
 }
 
-inline void write_to_file(const std::string &base, const std::string& path, const char* data, ssize_t size) {
+inline void write_to_file(const std::string &base, const std::string& path, const char* data, size_t size) {
     std::string fullpath = base + path;
 #ifdef _WIN32
-    std::wstring wpath = converter.from_bytes(fullpath);
+    std::wstring wpath = utf8ToUtf16(fullpath);
     auto fd = _wopen(wpath.c_str(), _O_WRONLY | _O_CREAT | _O_TRUNC, S_IWRITE);
 #else
     auto fd = open(fullpath.c_str(), O_WRONLY | O_CREAT | O_TRUNC,
@@ -259,7 +279,7 @@ inline void write_to_file(const std::string &base, const std::string& path, cons
         write_to_error_directory(base, path, data, size);
         return ;
     }
-    if (write(fd, data, size) != size) {
+    if ((size_t) write(fd, data, size) != size) {
       write_to_error_directory(base, path, data, size);
     }
     close(fd);
@@ -275,7 +295,7 @@ void ZimDumper::dumpFiles(const std::string& directory, bool symlinkdump, std::f
 {
   unsigned int truncatedFiles = 0;
 #if defined(_WIN32)
-    std::wstring wdir = converter.from_bytes(directory);
+    std::wstring wdir = utf8ToUtf16(directory);
     CreateDirectoryW(wdir.c_str(), NULL);
 #else
   ::mkdir(directory.c_str(), 0777);
@@ -366,7 +386,11 @@ Return value:
       See DIR/dump_errors.log for the listing of the errors.
 )";
 
-int subcmdInfo(ZimDumper &app, std::map<std::string, docopt::value> &args)
+
+// Older version of docopt doesn't define Options
+using Options = std::map<std::string, docopt::value>;
+
+int subcmdInfo(ZimDumper &app, Options &args)
 {
     app.printInfo();
     return 0;
@@ -382,7 +406,7 @@ int subcmdDumpAll(ZimDumper &app, const std::string &outdir, bool redirect, std:
     return 0;
 }
 
-int subcmdDump(ZimDumper &app,  std::map<std::string, docopt::value> &args)
+int subcmdDump(ZimDumper &app,  Options &args)
 {
     bool redirect = args["--redirect"].asBool();
 
@@ -405,22 +429,33 @@ int subcmdDump(ZimDumper &app,  std::map<std::string, docopt::value> &args)
     return subcmdDumpAll(app, directory, redirect, filter);
 }
 
-int subcmdShow(ZimDumper &app,  std::map<std::string, docopt::value> &args)
+zim::Entry getEntry(ZimDumper &app, Options &args)
+{
+    if (args["--idx"]) {
+        return app.getEntry(args["--idx"].asLong());
+    }
+
+    const std::string entryPath = args["--url"].asString();
+    const auto ns = args["--ns"];
+    if ( !ns ) {
+        return app.getEntryByPath(entryPath);
+    }
+
+    return app.getEntryByNsAndPath(ns.asString()[0], entryPath);
+}
+
+int subcmdShow(ZimDumper &app, Options &args)
 {
     // docopt guaranty us that we have `--idx` or `--url`.
     try {
-        if (args["--idx"]) {
-            return app.dumpEntry(app.getEntry(args["--idx"].asLong()));
-        } else {
-            return app.dumpEntry(app.getEntryByPath(args["--url"].asString()));
-        }
+        return app.dumpEntry(getEntry(app, args));
     } catch(...) {
         std::cerr << "Entry not found" << std::endl;
         return -1;
     }
 }
 
-int subcmdList(ZimDumper &app, std::map<std::string, docopt::value> &args)
+int subcmdList(ZimDumper &app, Options &args)
 {
     bool idx(args["--idx"]);
     bool url(args["--url"]);
@@ -451,8 +486,7 @@ int main(int argc, char* argv[])
     int ret = 0;
     std::ostringstream versions;
     printVersions(versions);
-    std::map<std::string, docopt::value> args
-        = docopt::docopt(USAGE,
+    Options args = docopt::docopt(USAGE,
                          { argv + 1, argv + argc },
                          true,
                          versions.str());
