@@ -19,15 +19,16 @@
 
 #include <iostream>
 #include <zim/search.h>
-#include <zim/file.h>
+#include <zim/archive.h>
 
 #include "version.h"
 
-void zimSearch(zim::Search& search)
+void printSearchResults(zim::Search& search)
 {
-    for (zim::Search::iterator it = search.begin(); it != search.end(); ++it)
+    auto results = search.getResults(0, search.getEstimatedMatches());
+    for (auto it = results.begin(); it != results.end(); ++it)
     {
-      std::cout << "article " << it->getIndex() << "\nscore " << it.get_score() << "\t:\t" << it->getTitle() << std::endl;
+      std::cout << "article " << it->getIndex() << "\nscore " << it.getScore() << "\t:\t" << it.getTitle() << std::endl;
     }
 }
 
@@ -37,9 +38,9 @@ int main(int argc, char* argv[])
   {
 
     // version number
-    if (argc > 1 && std::string(argv[1]) == "-v")
+    if (argc > 1 && (std::string(argv[1]) == "-v" || std::string(argv[1])=="--version"))
     {
-      version();
+      printVersions();
       return 0;
     }
 
@@ -50,7 +51,7 @@ int main(int argc, char* argv[])
         "\n"
         "options\n"
         "  -x indexfile   specify indexfile\n"
-        "  -v             print software version\n" << std::endl;
+        "  -v, --version  print software version\n" << std::endl;
       return 1;
     }
 
@@ -61,10 +62,12 @@ int main(int argc, char* argv[])
       s += argv[a];
     }
 
-    zim::File zimfile(argv[1]);
-    zim::Search search = zim::Search(&zimfile);
-    search.set_query(s);
-    zimSearch(search);
+    zim::Archive zimarchive(argv[1]);
+    zim::Searcher searcher(zimarchive);
+    zim::Query query;
+    query.setQuery(s);
+    auto search = searcher.search(query);
+    printSearchResults(search);
   }
   catch (const std::exception& e)
   {
