@@ -65,6 +65,7 @@ bool verboseFlag = false;
 bool withoutFTIndex = false;
 bool noUuid = false;
 bool dontCheckArgs = false;
+bool continue_without_magic = false;
 
 bool thereAreMissingArguments()
 {
@@ -75,10 +76,12 @@ bool thereAreMissingArguments()
     return false;
 
   return creator.empty()
+      || name.empty()
       || publisher.empty()
       || description.empty()
       || language.empty()
       || welcome.empty()
+      || title.empty()
       || illustration.empty();
 }
 
@@ -123,7 +126,11 @@ bool checkMetadata(const zim::Metadata& metadata)
 void addMetadata(ZimCreatorFS& zimCreator, const zim::Metadata& metadata)
 {
   for ( const auto& kv : metadata ) {
-    zimCreator.addMetadata(kv.first, kv.second);
+    if (kv.first == "Illustration_48x48@1") {
+      zimCreator.addIllustration(48, kv.second);
+    } else {
+      zimCreator.addMetadata(kv.first, kv.second);
+    }
   }
 }
 
@@ -145,6 +152,13 @@ bool isVerbose()
   return retVal;
 }
 
+void printZimWriterFsVersions(std::ostream& out = std::cout) {
+  out << "zim-tools " << VERSION << std::endl;
+  out << "+ libmagic " << magic_version() << std::endl;
+  out << std::endl;
+  zim::printVersions(out);
+}
+
 /* Print correct console usage options */
 void usage()
 {
@@ -168,11 +182,12 @@ void usage()
             << std::endl;
   std::cout << "\t-l, --language\t\tlanguage code of the content in ISO639-3"
             << std::endl;
+  std::cout << "\t-n, --name\t\tcustom (version independent) identifier for "
+               "the content"
+            << std::endl;
   std::cout << "\t-t, --title\t\ttitle of the ZIM file" << std::endl;
   std::cout << "\t-d, --description\tshort description of the content"
             << std::endl;
-  std::cout << "\t-L, --longDescription\tlong description of the content"
-            << std::endl;
   std::cout << "\t-c, --creator\t\tcreator(s) of the content" << std::endl;
   std::cout << "\t-p, --publisher\t\tcreator of the ZIM file itself"
             << std::endl;
@@ -189,6 +204,8 @@ void usage()
             << std::endl;
   std::cout << "\t-h, --help\t\tprint this help" << std::endl;
   std::cout << "\t-V, --version\t\tprint the version number" << std::endl;
+  std::cout << "\t-L, --longDescription\tlong description of the content"
+            << std::endl;
   std::cout
       << "\t-m, --clusterSize\tnumber of bytes per ZIM cluster (default: 2048Kb)"
       << std::endl;
@@ -205,13 +222,11 @@ void usage()
       << std::endl;
   std::cout << "\t-a, --tags\t\ttags - semicolon separated" << std::endl;
   std::cout << "\t-e, --source\t\tcontent source URL" << std::endl;
-  std::cout << "\t-n, --name\t\tcustom (version independent) identifier for "
-               "the content"
-            << std::endl;
   std::cout << "\t-o, --flavour\t\tcustom (version independent) content flavour"
             << std::endl;
   std::cout << "\t-s, --scraper\t\tname & version of tool used to produce HTML content"
             << std::endl;
+  std::cout << "\t--skip-libmagic-check\tAccept to run even if magic file cannot be loaded (mimetypes in the zim file may be wrong)." << std::endl;
   // --no-uuid and --dont-check-arguments are dev options, let's keep them secret
   // std::cout << "\t-U, --no-uuid\t\tdon't generate a random UUID" << std::endl;
   // std::cout << "\t-B, --dont-check-arguments\t\tdon't check arguments (and possibly produce a broken ZIM file)" << std::endl;
@@ -259,6 +274,7 @@ void parse_args(int argc, char** argv)
          {"threads", required_argument, 0, 'J'},
          {"no-uuid", no_argument, 0, 'U'},
          {"dont-check-arguments", no_argument, 0, 'B'},
+         {"skip-libmagic-check", no_argument, 0, 'M'},
 
          // Only for backward compatibility
          {"withFullTextIndex", no_argument, 0, 'i'},
@@ -269,7 +285,7 @@ void parse_args(int argc, char** argv)
 
   do {
     c = getopt_long(
-        argc, argv, "hVvijxuw:I:t:d:c:l:p:r:e:n:m:J:UBL:", long_options, &option_index);
+        argc, argv, "a:hVvijxuw:I:t:d:c:l:p:r:e:n:m:J:UBL:", long_options, &option_index);
 
     if (c != -1) {
       switch (c) {
@@ -277,7 +293,7 @@ void parse_args(int argc, char** argv)
           tags = optarg;
           break;
         case 'V':
-          printVersions();
+          printZimWriterFsVersions();
           exit(0);
           break;
         case 'h':
@@ -347,6 +363,9 @@ void parse_args(int argc, char** argv)
         case 'B':
           dontCheckArgs = true;
           break;
+        case 'M':
+          continue_without_magic = true;
+          break;
       }
     }
   } while (c != -1);
@@ -471,14 +490,19 @@ void create_zim(const zim::Metadata& metadata)
 /* Main program entry point */
 int main(int argc, char** argv)
 {
+  parse_args(argc, argv);
+
   /* Init */
   magic = magic_open(MAGIC_MIME);
-  magic_load(magic, NULL);
+  if (magic_load(magic, NULL) != 0) {
+    std::cerr << "Impossible to load magic file. Set `MAGIC` environment variable to a `magic` (or `magic.mgc`) file." << std::endl;
+    if (! continue_without_magic) {
+      exit(1);
+    }
+  }
   pthread_mutex_init(&verboseMutex, NULL);
 
   try {
-    parse_args(argc, argv);
-
     const zim::Metadata metadata = makeMetadata();
 
     if ( !checkMetadata(metadata) ) {
