@@ -131,41 +131,39 @@ struct CapturedStderr : CapturedStdStream
 int zimcheck (const std::vector<const char*>& args);
 
 const std::string zimcheck_help_message(
-  "\n"
-  "zimcheck checks the quality of a ZIM file.\n\n"
-  "Usage: zimcheck [options] zimfile\n"
-  "options:\n"
-  "-A , --all             run all tests. Default if no flags are given.\n"
-  "-0 , --empty           Empty content\n"
-  "-C , --checksum        Internal CheckSum Test\n"
-  "-I , --integrity       Low-level correctness/integrity checks\n"
-  "-M , --metadata        MetaData Entries\n"
-  "-F , --favicon         Favicon\n"
-  "-P , --main            Main page\n"
-  "-R , --redundant       Redundant data check\n"
-  "-U , --url_internal    URL check - Internal URLs\n"
-  "-X , --url_external    URL check - External URLs\n"
-  "-D , --details         Details of error\n"
-  "-B , --progress        Print progress report\n"
-  "-J , --json            Output in JSON format\n"
-  "-H , --help            Displays Help\n"
-  "-V , --version         Displays software version\n"
-  "-L , --redirect_loop   Checks for the existence of redirect loops\n"
-  "-W , --threads         count of threads to utilize (default: 1)\n"
-  "examples:\n"
-  "zimcheck -A wikipedia.zim\n"
-  "zimcheck --checksum --redundant wikipedia.zim\n"
-  "zimcheck -F -R wikipedia.zim\n"
-  "zimcheck -M --favicon wikipedia.zim\n"
-);
+R"(Zimcheck checks the quality of a ZIM file.
+
+Usage:
+  zimcheck [options] [ZIMFILE]
+
+Options:
+ -A --all             run all tests. Default if no flags are given.
+ -0 --empty           Empty content
+ -C --checksum        Internal CheckSum Test
+ -I --integrity       Low-level correctness/integrity checks
+ -M --metadata        MetaData Entries
+ -F --favicon         Favicon
+ -P --main            Main page
+ -R --redundant       Redundant data check
+ -U --url_internal    URL check - Internal URLs
+ -X --url_external    URL check - External URLs
+ -D --details         Details of error
+ -B --progress        Print progress report
+ -J --json            Output in JSON format
+ -H --help            Displays Help
+ -V --version         Displays software version
+ -L --redirect_loop   Checks for the existence of redirect loops
+ -W=<nb_thread> --threads=<nb_thread>  count of threads to utilize [default: 1]
+
+Examples:
+ zimcheck -A wikipedia.zim
+ zimcheck --checksum --redundant wikipedia.zim
+ zimcheck -F -R wikipedia.zim
+ zimcheck -M --favicon wikipedia.zim
+)");
 
 TEST(zimcheck, help)
 {
-    {
-      CapturedStdout zimcheck_output;
-      ASSERT_EQ(-1, zimcheck({"zimcheck", "-h"}));
-      ASSERT_EQ(zimcheck_help_message, std::string(zimcheck_output));
-    }
     {
       CapturedStdout zimcheck_output;
       ASSERT_EQ(-1, zimcheck({"zimcheck", "-H"}));
@@ -182,11 +180,6 @@ TEST(zimcheck, version)
 {
     std::string version = "zim-tools " + std::string(VERSION);
 
-    {
-      CapturedStdout zimcheck_output;
-      ASSERT_EQ(0, zimcheck({"zimcheck", "-v"}));
-      ASSERT_EQ(version, getLine(std::string(zimcheck_output)));
-    }
     {
       CapturedStdout zimcheck_output;
       ASSERT_EQ(0, zimcheck({"zimcheck", "-V"}));
@@ -261,7 +254,7 @@ TEST(zimcheck, integrity_goodzimfile)
     );
 
     test_zimcheck_single_option(
-        {"-i", "-I", "--integrity"},
+        {"-I", "--integrity"},
         GOOD_ZIMFILE,
         0,
         expected_output,
@@ -274,13 +267,14 @@ TEST(zimcheck, checksum_goodzimfile)
     const std::string expected_output(
         "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
         "[INFO] Zimcheck version is " VERSION "\n"
+        "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
         "[INFO] Verifying Internal Checksum..." "\n"
         "[INFO] Overall Test Status: Pass" "\n"
         "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
 
     test_zimcheck_single_option(
-        {"-c", "-C", "--checksum"},
+        {"-C", "--checksum"},
         GOOD_ZIMFILE,
         0,
         expected_output,
@@ -293,13 +287,14 @@ TEST(zimcheck, metadata_goodzimfile)
     const std::string expected_output(
         "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
         "[INFO] Zimcheck version is " VERSION "\n"
+        "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
         "[INFO] Checking metadata..." "\n"
         "[INFO] Overall Test Status: Pass" "\n"
         "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
 
     test_zimcheck_single_option(
-        {"-m", "-M", "--metadata"},
+        {"-M", "--metadata"},
         GOOD_ZIMFILE,
         0,
         expected_output,
@@ -312,13 +307,14 @@ TEST(zimcheck, favicon_goodzimfile)
     const std::string expected_output(
         "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
         "[INFO] Zimcheck version is " VERSION "\n"
+        "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
         "[INFO] Searching for Favicon..." "\n"
         "[INFO] Overall Test Status: Pass" "\n"
         "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
 
     test_zimcheck_single_option(
-        {"-f", "-F", "--favicon"},
+        {"-F", "--favicon"},
         GOOD_ZIMFILE,
         0,
         expected_output,
@@ -331,13 +327,14 @@ TEST(zimcheck, mainpage_goodzimfile)
     const std::string expected_output(
         "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
         "[INFO] Zimcheck version is " VERSION "\n"
+        "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
         "[INFO] Searching for main page..." "\n"
         "[INFO] Overall Test Status: Pass" "\n"
         "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
 
     test_zimcheck_single_option(
-        {"-p", "-P", "--main"},
+        {"-P", "--main"},
         GOOD_ZIMFILE,
         0,
         expected_output,
@@ -350,6 +347,7 @@ TEST(zimcheck, article_content_goodzimfile)
     const std::string expected_output(
         "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
         "[INFO] Zimcheck version is " VERSION "\n"
+        "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
         "[INFO] Verifying Articles' content..." "\n"
         "[INFO] Overall Test Status: Pass" "\n"
         "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
@@ -358,8 +356,8 @@ TEST(zimcheck, article_content_goodzimfile)
     test_zimcheck_single_option(
         {
           "-0", "--empty",              // Any of these options triggers
-          "-u", "-U", "--url_internal", // checking of the article contents.
-          "-x", "-X", "--url_external"  // For a good ZIM file there is no
+          "-U", "--url_internal", // checking of the article contents.
+          "-X", "--url_external"  // For a good ZIM file there is no
         },                              // difference in the output.
         GOOD_ZIMFILE,
         0,
@@ -373,6 +371,7 @@ TEST(zimcheck, redundant_articles_goodzimfile)
     const std::string expected_output(
         "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
         "[INFO] Zimcheck version is " VERSION "\n"
+        "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
         "[INFO] Verifying Articles' content..." "\n"
         "[INFO] Searching for redundant articles..." "\n"
         "  Verifying Similar Articles for redundancies..." "\n"
@@ -381,7 +380,7 @@ TEST(zimcheck, redundant_articles_goodzimfile)
     );
 
     test_zimcheck_single_option(
-        {"-r", "-R", "--redundant"},
+        {"-R", "--redundant"},
         GOOD_ZIMFILE,
         0,
         expected_output,
@@ -394,13 +393,14 @@ TEST(zimcheck, redirect_loop_goodzimfile)
   const std::string expected_output(
     "[INFO] Checking zim file data/zimfiles/good.zim" "\n"
     "[INFO] Zimcheck version is " VERSION "\n"
+    "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
     "[INFO] Checking for redirect loops..." "\n"
     "[INFO] Overall Test Status: Pass" "\n"
     "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
   );
 
   test_zimcheck_single_option(
-    {"-l", "-L", "--redirect_loop"},
+    {"-L", "--redirect_loop"},
     GOOD_ZIMFILE,
     0,
     expected_output,
@@ -435,7 +435,7 @@ TEST(zimcheck, nooptions_goodzimfile)
 TEST(zimcheck, all_checks_goodzimfile)
 {
     test_zimcheck_single_option(
-        {"-a", "-A", "--all"},
+        {"-A", "--all"},
         GOOD_ZIMFILE,
         0,
         ALL_CHECKS_OUTPUT_ON_GOODZIMFILE,
@@ -448,8 +448,8 @@ TEST(zimcheck, invalid_option)
     {
       CapturedStdout zimcheck_output;
       CapturedStderr zimcheck_stderr;
-      ASSERT_EQ(1, zimcheck({"zimcheck", "-z", GOOD_ZIMFILE}));
-      ASSERT_EQ("Unknown option `-z'\n", std::string(zimcheck_stderr));
+      ASSERT_EQ(1, zimcheck({"zimcheck", "-Z", GOOD_ZIMFILE}));
+      ASSERT_EQ("Unexpected argument: -Z, data/zimfiles/good.zim\n", std::string(zimcheck_stderr));
       ASSERT_EQ(zimcheck_help_message, std::string(zimcheck_output));
     }
 }
@@ -460,7 +460,7 @@ TEST(zimcheck, invalid_long_option)
       CapturedStdout zimcheck_output;
       CapturedStderr zimcheck_stderr;
       ASSERT_EQ(1, zimcheck({"zimcheck", "--oops", GOOD_ZIMFILE}));
-      ASSERT_EQ("Unknown option `--oops'\n", std::string(zimcheck_stderr));
+      ASSERT_EQ("Unexpected argument: --oops, data/zimfiles/good.zim\n", std::string(zimcheck_stderr));
       ASSERT_EQ(zimcheck_help_message, std::string(zimcheck_output));
     }
 }
@@ -487,6 +487,7 @@ TEST(zimcheck, json_goodzimfile)
       "    \"redundant\","                                          "\n"
       "    \"url_internal\","                                       "\n"
       "    \"url_external\","                                       "\n"
+      "    \"url_empty\","                                          "\n"
       "    \"redirect\""                                            "\n"
       "  ],"                                                        "\n"
       "  \"file_name\" : \"data/zimfiles/good.zim\","               "\n"
@@ -504,6 +505,7 @@ TEST(zimcheck, bad_checksum)
     const std::string expected_output(
       "[INFO] Checking zim file data/zimfiles/bad_checksum.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Verifying Internal Checksum..." "\n"
       "  [ERROR] Wrong Checksum in ZIM archive" "\n"
       "[ERROR] Invalid checksum:" "\n"
@@ -514,7 +516,7 @@ TEST(zimcheck, bad_checksum)
     );
 
     test_zimcheck_single_option(
-        {"-c", "-C", "--checksum"},
+        {"-C", "--checksum"},
         BAD_CHECKSUM_ZIMFILE,
         1,
         expected_output,
@@ -527,19 +529,20 @@ TEST(zimcheck, metadata_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Checking metadata..." "\n"
       "[ERROR] Metadata errors:" "\n"
       "  Missing mandatory metadata: Title" "\n"
       "  Missing mandatory metadata: Description" "\n"
       "  Missing mandatory metadata: Illustration_48x48@1" "\n"
       "  Language must contain at least 3 characters" "\n"
-      "  Language doesn't match regex: \\w{3}(,\\w{3})*" "\n"
+      "  Language doesn't match regex: ^\\w{3}(,\\w{3})*$" "\n"
       "[INFO] Overall Test Status: Fail" "\n"
       "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
 
     test_zimcheck_single_option(
-        {"-m", "-M", "--metadata"},
+        {"-M", "--metadata"},
         POOR_ZIMFILE,
         1,
         expected_stdout,
@@ -552,6 +555,7 @@ TEST(zimcheck, favicon_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Searching for Favicon..." "\n"
       "[ERROR] Favicon:" "\n"
       "  Favicon is missing" "\n"
@@ -560,7 +564,7 @@ TEST(zimcheck, favicon_poorzimfile)
     );
 
     test_zimcheck_single_option(
-        {"-f", "-F", "--favicon"},
+        {"-F", "--favicon"},
         POOR_ZIMFILE,
         1,
         expected_stdout,
@@ -573,6 +577,7 @@ TEST(zimcheck, mainpage_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Searching for main page..." "\n"
       "[ERROR] Missing mainpage:" "\n"
       "  Main Page Index stored in Archive Header: 4294967295" "\n"
@@ -581,7 +586,7 @@ TEST(zimcheck, mainpage_poorzimfile)
     );
 
     test_zimcheck_single_option(
-        {"-p", "-P", "--main"},
+        {"-P", "--main"},
         POOR_ZIMFILE,
         1,
         expected_stdout,
@@ -594,6 +599,7 @@ TEST(zimcheck, empty_items_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Verifying Articles' content..." "\n"
       "[ERROR] Empty articles:" "\n"
       "  Entry empty.html is empty" "\n"
@@ -615,19 +621,21 @@ TEST(zimcheck, internal_url_check_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Verifying Articles' content..." "\n"
       "[ERROR] Invalid internal links found:" "\n"
       "  The following links:" "\n"
       "- A/non_existent.html" "\n"
       "(A/non_existent.html) were not found in article dangling_link.html" "\n"
-      "  Found 1 empty links in article: empty_link.html" "\n"
       "  ../../oops.html is out of bounds. Article: outofbounds_link.html" "\n"
+      "[WARNING] Empty links found:" "\n"
+      "  Found 1 empty links in article: empty_link.html" "\n"
       "[INFO] Overall Test Status: Fail" "\n"
       "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
 
     test_zimcheck_single_option(
-        {"-u", "-U", "--url_internal"},
+        {"-U", "--url_internal"},
         POOR_ZIMFILE,
         1,
         expected_stdout,
@@ -640,6 +648,7 @@ TEST(zimcheck, external_url_check_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Verifying Articles' content..." "\n"
       "[ERROR] Invalid external links found:" "\n"
       "  http://a.io/pic.png is an external dependence in article external_image_http.html" "\n"
@@ -650,7 +659,7 @@ TEST(zimcheck, external_url_check_poorzimfile)
     );
 
     test_zimcheck_single_option(
-        {"-x", "-X", "--url_external"},
+        {"-X", "--url_external"},
         POOR_ZIMFILE,
         1,
         expected_stdout,
@@ -663,6 +672,7 @@ TEST(zimcheck, redundant_poorzimfile)
     const std::string expected_stdout(
       "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
       "[INFO] Zimcheck version is " VERSION "\n"
+      "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
       "[INFO] Verifying Articles' content..." "\n"
       "[INFO] Searching for redundant articles..." "\n"
       "  Verifying Similar Articles for redundancies..." "\n"
@@ -673,7 +683,7 @@ TEST(zimcheck, redundant_poorzimfile)
     );
 
     test_zimcheck_single_option(
-        {"-r", "-R", "--redundant"},
+        {"-R", "--redundant"},
         POOR_ZIMFILE,
         0,
         expected_stdout,
@@ -686,6 +696,7 @@ TEST(zimcheck, redirect_loop_poorzimfile)
   const std::string expected_output(
     "[INFO] Checking zim file data/zimfiles/poor.zim" "\n"
     "[INFO] Zimcheck version is " VERSION "\n"
+    "[WARNING] Integrity check is skipped. Any detected errors may in fact be due to corrupted/invalid data.\n"
     "[INFO] Checking for redirect loops..." "\n"
     "[ERROR] Redirect loop(s) exist:" "\n"
     "  Redirect loop exists from entry redirect_loop.html" "\n"
@@ -699,7 +710,7 @@ TEST(zimcheck, redirect_loop_poorzimfile)
   );
 
   test_zimcheck_single_option(
-    {"-l", "-L", "--redirect_loop"},
+    {"-L", "--redirect_loop"},
     POOR_ZIMFILE,
     1,
     expected_output,
@@ -726,7 +737,7 @@ const std::string ALL_CHECKS_OUTPUT_ON_POORZIMFILE(
       "  Missing mandatory metadata: Description" "\n"
       "  Missing mandatory metadata: Illustration_48x48@1" "\n"
       "  Language must contain at least 3 characters" "\n"
-      "  Language doesn't match regex: \\w{3}(,\\w{3})*" "\n"
+      "  Language doesn't match regex: ^\\w{3}(,\\w{3})*$" "\n"
       "[ERROR] Favicon:" "\n"
       "  Favicon is missing" "\n"
       "[ERROR] Missing mainpage:" "\n"
@@ -737,12 +748,13 @@ const std::string ALL_CHECKS_OUTPUT_ON_POORZIMFILE(
       "  The following links:" "\n"
       "- A/non_existent.html" "\n"
       "(A/non_existent.html) were not found in article dangling_link.html" "\n"
-      "  Found 1 empty links in article: empty_link.html" "\n"
       "  ../../oops.html is out of bounds. Article: outofbounds_link.html" "\n"
       "[ERROR] Invalid external links found:" "\n"
       "  http://a.io/pic.png is an external dependence in article external_image_http.html" "\n"
       "  https://a.io/pic.png is an external dependence in article external_image_https.html" "\n"
       "  //a.io/pic.png is an external dependence in article external_image_protocol_relative.html" "\n"
+      "[WARNING] Empty links found:" "\n"
+      "  Found 1 empty links in article: empty_link.html" "\n"
       "[ERROR] Redirect loop(s) exist:" "\n"
       "  Redirect loop exists from entry redirect_loop.html" "\n"
       "" "\n"
@@ -765,7 +777,7 @@ TEST(zimcheck, nooptions_poorzimfile)
 TEST(zimcheck, all_checks_poorzimfile)
 {
     test_zimcheck_single_option(
-        {"-a", "-A", "--all"},
+        {"-A", "--all"},
         POOR_ZIMFILE,
         1,
         ALL_CHECKS_OUTPUT_ON_POORZIMFILE,
@@ -823,6 +835,7 @@ TEST(zimcheck, json_poorzimfile)
       "    \"redundant\","                                                  "\n"
       "    \"url_internal\","                                               "\n"
       "    \"url_external\","                                               "\n"
+      "    \"url_empty\","                                                  "\n"
       "    \"redirect\""                                                    "\n"
       "  ],"                                                                "\n"
       "  \"file_name\" : \"data/zimfiles/poor.zim\","                       "\n"
@@ -862,8 +875,8 @@ TEST(zimcheck, json_poorzimfile)
       "    {"                                                               "\n"
       "      \"check\" : \"metadata\","                                     "\n"
       "      \"level\" : \"ERROR\","                                        "\n"
-      "      \"message\" : \"Language doesn't match regex: \\\\w{3}(,\\\\w{3})*\"," "\n"
-      "      \"error\" : \"Language doesn't match regex: \\\\w{3}(,\\\\w{3})*\""    "\n"
+      "      \"message\" : \"Language doesn't match regex: ^\\\\w{3}(,\\\\w{3})*$\"," "\n"
+      "      \"error\" : \"Language doesn't match regex: ^\\\\w{3}(,\\\\w{3})*$\""    "\n"
       "    },"                                                              "\n"
       "    {"                                                               "\n"
       "      \"check\" : \"favicon\","                                      "\n"
@@ -896,13 +909,6 @@ TEST(zimcheck, json_poorzimfile)
       "    {"                                                               "\n"
       "      \"check\" : \"url_internal\","                                 "\n"
       "      \"level\" : \"ERROR\","                                        "\n"
-      "      \"message\" : \"Found 1 empty links in article: empty_link.html\"," "\n"
-      "      \"count\" : \"1\","                                            "\n"
-      "      \"path\" : \"empty_link.html\""                                "\n"
-      "    },"                                                              "\n"
-      "    {"                                                               "\n"
-      "      \"check\" : \"url_internal\","                                 "\n"
-      "      \"level\" : \"ERROR\","                                        "\n"
       "      \"message\" : \"../../oops.html is out of bounds. Article: outofbounds_link.html\"," "\n"
       "      \"link\" : \"../../oops.html\","                               "\n"
       "      \"path\" : \"outofbounds_link.html\""                          "\n"
@@ -929,6 +935,13 @@ TEST(zimcheck, json_poorzimfile)
       "      \"path\" : \"external_image_protocol_relative.html\""                             "\n"
       "    },"                                                              "\n"
       "    {"                                                               "\n"
+      "      \"check\" : \"url_empty\","                                    "\n"
+      "      \"level\" : \"WARNING\","                                      "\n"
+      "      \"message\" : \"Found 1 empty links in article: empty_link.html\"," "\n"
+      "      \"count\" : \"1\","                                            "\n"
+      "      \"path\" : \"empty_link.html\""                                "\n"
+      "    },"                                                              "\n"    
+      "    {"                                                               "\n"
       "      \"check\" : \"redirect\","                                     "\n"
       "      \"level\" : \"ERROR\","                                        "\n"
       "      \"message\" : \"Redirect loop exists from entry redirect_loop.html\\n\"," "\n"
